

void users_init_mode(void)
{
// A switch statement can be used to do different things depending on
// the processing mode.
switch (rx_mode)
  {
  case MODE_WCW:
  case MODE_HSMS:
  case MODE_QRSS:
  break;
  
  case MODE_NCW:
  case MODE_SSB:
// Open a window to allow mouse control of your own things
  init_users_control_window();  
  break;
  
  case MODE_FM:
  case MODE_AM:
  case MODE_TXTEST:
  case MODE_ADTEST:
  case MODE_TUNE:
  break;
  }
}




// Prototype for user defined hardware drive routines.
// These routines will replace the routines in hwaredriver.c
// in case a routine users_hwaredriver.c is present when configure is run.
// The Linrad package does not contain a users_hwaredriver.c file
// so it is safe to copy a nev version on top of an old one.
// The users_hwaredriver.c file will not be overwritten.

void users_eme(void)
{
// Called each time moon position is updated.
// data in degrees is available in:
// float moon_az;
// float moon_el;
}

void hware_interface_test(void)
{
// This routine is called from the main menu.
// Here you can put test routines for your hardware interface.
}


void userdefined_u(void)
{
// This routine is called when the 'U' key is pressed
}

void userdefined_q(void)
{
// This routine is called when the 'Q' key is pressed
}


void users_open_devices(void)
{
// Open ports as required
// set allow_parport=1 if the parallel port is allowed.
// We read this port now and then if allowed to in order to
// allow Linrad run without having apm shutting down the computer.
}

void set_hardware_rx_gain(int band_no)
{
// Do something with band_no to keep the compiler happy.
int dummy;
dummy=band_no;
// Input data:
// int fg.gain
// int band_no
// The argument band_no will allow different hardware on different
// bands. 
// Set hardware gain
// This is the data entered in the rx gain control box.
// This routine is responsible for checking that it is compatible to
// hardware capabilities and adjusting it accordingly.
// this routine is also responsible for sending the control information
// to the hardware. Either by setting hware_flag=1 which will
// cause calls to control_hware() until hware_flag is cleared.
// (look at hwaredriver.c to see how it works to make serial data
// on the parallel port).
// In case the hardware can be controlled by the serial port, 
// just write the control informatio to the serial port here.
//************************************************************
// This routine is responsible for setting
// int fg.gain_increment = The increment in dB for clicking
// the arrows in the gain control box.
}

void set_hardware_rx_frequency(void)
{
// Input data:
// double fg.passband_center = The midpoint of the waterfall in MHz.
// This is the data entered in the rx frequency control box.
// This routine is responsible for checking that it is compatible to
// hardware capabilities and adjusting it accordingly.
// this routine is also responsible for sending the control information
// to the hardware. Either by setting hware_flag=1 which will
// cause calls to control_hware() until hware_flag is cleared.
// (look at hwaredriver.c to see how it works to make serial data
// on the parallel port).
// In case the hardware can be controlled by the serial port, 
// just write the control informatio to the serial port here.
//************************************************************
// This routine is responsible for setting
// double fg.passband_increment = The increment in MHz for clicking
// the arrows in the rx control box.
//************************************************************
// This routine is responsible for setting
// int fg.passband_direction = -1 or +1 depending on whether the
// hardware local oscillator is above or below the desired signal. 
//************************************************************
// This routine is responsible for setting
// int fft1_direction = fg.passband_direction
}

void clear_hware_data(void)
{
// This routine can be used to stop the serial data on the
// parallel port so one does not have to wait for it to complete
// if a new frequency has been entered soon after a change.
// See hwaredriver.c
}

void control_hware(void)
{
// This routine is responsible to set
// int hware_flag=0 when it does not want to be called any more.
// If you use this routine to read data from your hardware
// to update fg.passband_center or fg.passband_direction,
// call show_center_frequency() to make Linrad update the frequency scales
// on the screen.  
}

//    ************ PROTOTYPE ROUTINES FOR USER WINDOWS  ***************
// User defined windows to allow mouse control.
// In case several windows are desired, give them different
// negative numbers and use scro[m].no to decide what to do
// in your mouse_on_users_graph routine.
#define USERS_GRAPH_TYPE1 -1
// Define your parameters in this structure.
// Save it to a file and recover it from init_users_control_window(void)

typedef struct {
int ytop;
int ybottom;
int xleft;
int xright;
int par1;
float par2;
} UG_PARMS;

#define UG_TOP 0
#define UG_BOTTOM 1
#define UG_LEFT 2
#define UG_RIGHT 3
#define UG_INCREASE_PAR1 4
#define UG_DECREASE_PAR1 5
#define UG_DO_WHATEVER 6
#define MAX_UGBUTT 7

UG_PARMS ug;
BUTTONS ugbutt[MAX_UGBUTT];

int ug_old_y1;
int ug_old_y2;
int ug_old_x1;
int ug_old_x2;
void make_users_control_graph(int m);

void ug_borders(int color)
{
errno=912546;
ck_hline(ug.xleft,ug.ytop,ug.xright,color);
errno=912545;
ck_hline(ug.xleft,ug.ybottom,ug.xright,color);
errno=912548;
ck_line(ug.xleft,ug.ytop,ug.xleft,ug.ybottom,color);
errno=912549;
ck_line(ug.xright,ug.ytop,ug.xright,ug.ybottom,color);
}

void show_user_parms(void)
{
char s[80];
// Show the user parameters on screen
// and issue hardware commands if required.
// Use hware_flag to direct calls to control_hware() when cpu time 
// is available in case whatever you want to do takes too much time 
// to be done immediately.
sprintf(s,"Par1 %d ",ug.par1);
errno=912700;
xytext(ug.xleft/text_width+6,ug.ytop/text_height+2,s);
sprintf(s,"Par2 %f ",ug.par2);
errno=912700;
xytext(ug.xleft/text_width+6,ug.ytop/text_height+1,s);
}


void mouse_continue_users_graph(void)
{
int i;
hide_mouse();
switch (mouse_active_flag-1)
  {
// Move border lines immediately.
// for other functions, wait until button is released.
// Look in freq_control.c how to move a fixed size window.
  case UG_TOP:
  ug_borders(0);
  ug.ytop=mouse_y;
  ug_borders(15);
  break;

  case UG_BOTTOM:
  ug_borders(0);
  ug.ybottom=mouse_y;
  ug_borders(15);
  break;
  
  case UG_LEFT:
  ug_borders(0);
  ug.xleft=mouse_x;
  ug_borders(15);
  break;
  
  case UG_RIGHT:
  ug_borders(0);
  ug.xright=mouse_x;
  ug_borders(15);
  break;
      
  default:
  goto await_release;
  }
if(leftpressed == 2)goto finish;
show_mouse();
return;
await_release:;
if(leftpressed == 1)
  {
  show_mouse();
  return;
  }
if(network_flag != 2)
  {
  switch (mouse_active_flag-1)
    {
    case UG_INCREASE_PAR1: 
    ug.par1++;
    break;
  
    case UG_DECREASE_PAR1:
    ug.par1--;
    break;

    case UG_DO_WHATEVER: 
// Issue hardware commands or fill the screen
// with whatever you like.
// Use hware_flag to direct calls to control_hware() when cpu time 
// is available in case whatever you want to do takes too much time 
// to be done immediately.
    break;
  
    default:
// This should never happen.    
    end_program(211053);
    break;
    }
  }  
finish:;
leftpressed=0;  
mouse_active_flag=0;
for(i=0; i<no_of_scro; i++)
  {
  if(scro[i].no == USERS_GRAPH_TYPE1)
    {
    ug_borders(0);
errno=40133;
    ck_fillbox(ug_old_x1,ug_old_y1,ug_old_x2-ug_old_x1,ug_old_y2-ug_old_y1,0);
    make_users_control_graph(i);
    show_mouse();
    return;
    }
  }
// We should never arrive here:-)
end_program(342878);  
}

void new_user_par1(void)
{
hide_mouse();
ug.par1=numinput_int_data;
show_user_parms();
leftpressed=0;
show_mouse();
}

void new_user_par2(void)
{
hide_mouse();
ug.par2=numinput_float_data;
show_user_parms();
leftpressed=0;
show_mouse();
}




void mouse_on_users_graph(void)
{
int event_no;
// First find out is we are on a button or border line.
for(event_no=0; event_no<MAX_UGBUTT; event_no++)
  {
  if( ugbutt[event_no].x1 <= mouse_x && 
      ugbutt[event_no].x2 >= mouse_x &&      
      ugbutt[event_no].y1 <= mouse_y && 
      ugbutt[event_no].y2 >= mouse_y) 
    {
    ug_old_y1=ug.ytop;
    ug_old_y2=ug.ybottom;
    ug_old_x1=ug.xleft;
    ug_old_x2=ug.xright;
    mouse_active_flag=1+event_no;
    current_mouse_activity=mouse_continue_users_graph;
    return;
    }
  }
// Not button or border.
// Prompt the user for par1 or par2 depending on whether the mouse is
// in the upper or the lower part of the window.
mouse_active_flag=1;
numinput_xpix=ug.xleft+11*text_width;
hide_mouse();
if(mouse_y < (ug.ytop+ug.ybottom)/2)
  {
  numinput_ypix=ug.ytop+2*text_height;
  numinput_chars=12;
  erase_numinput_txt();
  numinput_flag=FIXED_FLOAT_PARM;
  par_from_keyboard_routine=new_user_par2;
  }
else
  {
  numinput_ypix=ug.ytop+text_height;
  numinput_chars=4;
  erase_numinput_txt();
  numinput_flag=FIXED_INT_PARM;
  par_from_keyboard_routine=new_user_par1;
  }  
show_mouse();
}


void make_users_control_graph(int m)
{
// Set a negative number here.
// In case several windows are desired, give them different
// negative numbers and use scro[m].no to decide what to do
// in your mouse_on_users_graph routine.
scro[m].no=USERS_GRAPH_TYPE1;
// These are the coordinates of the border lines.
scro[m].x1=ug.xleft;
scro[m].x2=ug.xright;
scro[m].y1=ug.ytop;
scro[m].y2=ug.ybottom;
// Each border line is treated as a button.
// That is for the mouse to get hold of them so the window can be moved.
ugbutt[UG_LEFT].x1=ug.xleft;
ugbutt[UG_LEFT].x2=ug.xleft;
ugbutt[UG_LEFT].y1=ug.ytop;
ugbutt[UG_LEFT].y2=ug.ybottom;
ugbutt[UG_RIGHT].x1=ug.xright;
ugbutt[UG_RIGHT].x2=ug.xright;
ugbutt[UG_RIGHT].y1=ug.ytop;
ugbutt[UG_RIGHT].y2=ug.ybottom;
ugbutt[UG_TOP].x1=ug.xleft;
ugbutt[UG_TOP].x2=ug.xright;
ugbutt[UG_TOP].y1=ug.ytop;
ugbutt[UG_TOP].y2=ug.ytop;
ugbutt[UG_BOTTOM].x1=ug.xleft;
ugbutt[UG_BOTTOM].x2=ug.xright;
ugbutt[UG_BOTTOM].y1=ug.ybottom;
ugbutt[UG_BOTTOM].y2=ug.ybottom;
// Draw the border lines
ug_borders(7);
settextcolor(7);
make_button(ug.xleft+text_width,ug.ybottom-3*text_height/2-2,
                                         ugbutt,UG_DECREASE_PAR1,25);
make_button(ug.xright-text_width,ug.ybottom-3*text_height/2-2,
                                     ugbutt,UG_INCREASE_PAR1,24); 
make_button(ug.xleft+text_width,ug.ybottom-text_height/2-2,
                                         ugbutt,UG_DO_WHATEVER,'A');
show_user_parms();
}



void init_users_control_window(void)
{
// Set initial values in code or by reading
// a file of your own.
ug.xleft=0;
ug.xright=25*text_width;
ug.ytop=0;
ug.ybottom=3.5*text_height;
ug.par1=5;
ug.par2=PI_L;
make_users_control_graph(no_of_scro);
no_of_scro++;
if(no_of_scro >= MAX_SCRO)end_program(89);
}


